package com.gyee.frame.controller.ticket;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gyee.frame.common.dataSources.DataSource;
import com.gyee.frame.common.dataSources.DataSourceType;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.ticket.Wohaveele;
import com.gyee.frame.model.ticket.Wosafemeasure;
import com.gyee.frame.service.ticket.WohaveeleService;
import com.gyee.frame.service.ticket.WosafemeasureService;
import com.gyee.frame.util.ticket.TicketUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 电气二种工作票安全措施
 */

@RestController
@RequestMapping("/elec2/safe/measure")
public class Elec2SafeMeasureController {

    @Autowired
    WohaveeleService wohaveeleService;
    @Autowired
    WosafemeasureService wosafemeasureService;


    /**
     * 根据主单据的 wonum 关联编号查询安全措施
     *
     * @param wonum
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @GetMapping(value = "list")
    public AjaxResult getSafeMeasure(@RequestParam("wonum") String wonum) {
        // 工作条件(停电或不停电，或邻近及保留带电设备名称)
        List<Wohaveele> condition = wohaveeleService.getListSafeMeasure(wonum);
        // 安全措施
        List<Wosafemeasure> safety = wosafemeasureService.getListSafeMeasure(wonum, null);

        Map<String, Object> maps = new HashMap<>();
        maps.put("condition", condition);
        maps.put("safety", safety);

        return AjaxResult.successData(maps);
    }

    /**
     * 编辑安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "edit")
    public AjaxResult editSafeMeasure(@RequestParam("data") String data, @RequestParam("type") String type) {
        if (type.isEmpty())
            return AjaxResult.error(4004, "数据更新失败");

        if (type.equals("condition")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wohaveele measure = JSON.toJavaObject(jsonObject, Wohaveele.class);

            boolean result = wohaveeleService.updateById(measure);
            if (!result)
                return AjaxResult.error(4004, "数据更新失败");
        }
        if (type.equals("safety")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wosafemeasure measure = JSON.toJavaObject(jsonObject, Wosafemeasure.class);

            boolean result = wosafemeasureService.updateById(measure);
            if (!result)
                return AjaxResult.error(4004, "数据更新失败");
        }

        return AjaxResult.success("数据更新成功");
    }

    /**
     * 根据 wonum 新增安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "create")
    public AjaxResult createSafeMeasure(@RequestParam("data") String data, @RequestParam("type") String type) {
        if (type.isEmpty())
            return AjaxResult.error(4004, "数据插入失败");

        if (type.equals("condition")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wohaveele measure = JSON.toJavaObject(jsonObject, Wohaveele.class);
            int id = wohaveeleService.selectMaxId();
            measure.setId((double) (id + 1));

            boolean result = wohaveeleService.save(measure);
            if (!result)
                return AjaxResult.error(4004, "数据插入失败");
        }
        if (type.equals("safety")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wosafemeasure measure = JSON.toJavaObject(jsonObject, Wosafemeasure.class);
            int id = wosafemeasureService.selectMaxId();
            measure.setId((double) (id + 1));

            boolean result = wosafemeasureService.save(measure);
            if (!result)
                return AjaxResult.error(4004, "数据插入失败");
        }
        return AjaxResult.success("数据添加成功");
    }


    /**
     * 批量删除安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "delete")
    public AjaxResult deleteSafeMeasure(@RequestParam("list") String data) {
        Map map = TicketUtil.formatSafeMeasureElec2(data);
        List listC = (List) map.get("condition");
        List listS = (List) map.get("safety");

        if (listC != null && listC.size() > 0) {
            int code = wohaveeleService.deleteSafeMeasure(listC);
            if (code <= 0)
                return AjaxResult.error(4004, "数据删除失败");
        }
        if (listS != null && listS.size() > 0) {
            int code = wosafemeasureService.deleteSafeMeasure(listS);
            if (code <= 0)
                return AjaxResult.success("数据删除失败");
        }

        return AjaxResult.success("数据删除成功");
    }
}
