package com.gyee.frame.controller.ticket;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gyee.frame.common.dataSources.DataSource;
import com.gyee.frame.common.dataSources.DataSourceType;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.ticket.Woadditionsafemeasure;
import com.gyee.frame.model.ticket.Wosafety;
import com.gyee.frame.service.ticket.WoadditionsafemeasureService;
import com.gyee.frame.service.ticket.WosafetyService;
import com.gyee.frame.util.ticket.TicketUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 线路二种工作票安全措施
 */

@RestController
@RequestMapping("/line2/safe/measure")
public class Line2SafeMeasureController {

    @Autowired
    WosafetyService wosafetyService;
    @Autowired
    WoadditionsafemeasureService woadditionsafemeasureService;

    /**
     * 根据主单据的 wonum 关联编号查询安全措施
     *
     * @param wonum
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @GetMapping(value = "list")
    public AjaxResult getSafeMeasure(@RequestParam("wonum") String wonum) {
        // 工作条件(停电或不停电，或邻近及保留带电设备名称)
        List<Wosafety> condition = wosafetyService.getListSafeMeasure(wonum);
        // 安全措施
        List<Woadditionsafemeasure> safety = woadditionsafemeasureService.getListSafeMeasure(wonum);

        Map<String, Object> maps = new HashMap<>();
        maps.put("condition", condition);
        maps.put("safety", safety);

        return AjaxResult.successData(maps);
    }

    /**
     * 编辑安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "edit")
    public AjaxResult editSafeMeasure(@RequestParam("data") String data, @RequestParam("type") String type) {
        if (type.isEmpty())
            return AjaxResult.error(4004, "数据更新失败");

        if (type.equals("condition")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wosafety measure = JSON.toJavaObject(jsonObject, Wosafety.class);

            boolean result = wosafetyService.updateById(measure);
            if (!result)
                return AjaxResult.error(4004, "数据更新失败");
        }
        if (type.equals("safety")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Woadditionsafemeasure measure = JSON.toJavaObject(jsonObject, Woadditionsafemeasure.class);

            boolean result = woadditionsafemeasureService.updateById(measure);
            if (!result)
                return AjaxResult.error(4004, "数据更新失败");
        }

        return AjaxResult.success("数据更新成功");
    }

    /**
     * 根据 wonum 新增安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "create")
    public AjaxResult createSafeMeasure(@RequestParam("data") String data, @RequestParam("type") String type) {
        if (type.isEmpty())
            return AjaxResult.error(4004, "数据插入失败");

        if (type.equals("condition")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Wosafety measure = JSON.toJavaObject(jsonObject, Wosafety.class);
            int id = wosafetyService.selectMaxId();
            measure.setId((double) (id + 1));

            boolean result = wosafetyService.save(measure);
            if (!result)
                return AjaxResult.error(4004, "数据插入失败");
        }
        if (type.equals("safety")) {
            JSONObject jsonObject = JSONObject.parseObject(data);
            Woadditionsafemeasure measure = JSON.toJavaObject(jsonObject, Woadditionsafemeasure.class);
            int id = woadditionsafemeasureService.selectMaxId();
            measure.setId((double) (id + 1));

            boolean result = woadditionsafemeasureService.save(measure);
            if (!result)
                return AjaxResult.error(4004, "数据插入失败");
        }
        return AjaxResult.success("数据添加成功");
    }


    /**
     * 批量删除安全措施
     *
     * @param data
     * @return
     */
    @DataSource(value = DataSourceType.TICKET)
    @PostMapping(value = "delete")
    public AjaxResult deleteSafeMeasure(@RequestParam("list") String data) {
        Map map = TicketUtil.formatSafeMeasureLine2(data);
        List listC = (List) map.get("condition");
        List listS = (List) map.get("safety");

        if (listC != null && listC.size() > 0) {
            int code = wosafetyService.deleteSafeMeasure(listC);
            if (code <= 0)
                return AjaxResult.error(4004, "数据删除失败");
        }
        if (listS != null && listS.size() > 0) {
            int code = woadditionsafemeasureService.deleteSafeMeasure(listS);
            if (code <= 0)
                return AjaxResult.success("数据删除失败");
        }

        return AjaxResult.success("数据删除成功");
    }
}
